#!/bin/bash
#
# Copyright (c) 2007 Regents of the SIGNET lab, University of Padova.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University of Padova (SIGNET lab) nor the 
#    names of its contributors may be used to endorse or promote products 
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


##############################
# Configuration parameters
##############################

# change these according to your input video and the desired codec settings.
VIDEONAME=news
VIDEO_FORMAT=qcif
FRAME_RATE=25
GOP_SIZE=15
NUM_B_FRAMES=2



###################################
# End of configuration parameters
###################################




for tool in ffmpeg mp4trace
do
    if ! which $tool &> /dev/null ; then
	echo "Could not find $tool, exiting"
	exit 1
    fi
done





if test ! -f "${VIDEONAME}.yuv" ; then
    echo "Could not find raw yuv video file \"${VIDEONAME}.yuv\""
    exit 1
fi



ffmpeg -s ${VIDEO_FORMAT} -i ${VIDEONAME}.yuv -y -vcodec mpeg4 -r ${FRAME_RATE} -g ${GOP_SIZE} -bf ${NUM_B_FRAMES} ${VIDEONAME}.m4v

MP4Box -hint -mtu 1024 -fps ${FRAME_RATE} -add ${VIDEONAME}.m4v  ${VIDEONAME}.mp4

ffmpeg -i ${VIDEONAME}.mp4 -y -r ${FRAME_RATE}  ${VIDEONAME}_ref.yuv


echo > ${VIDEONAME}.dvs

echo
echo "Now open a new shell, become root and run the following command:"
echo
echo "tcpdump -n -tt -v -i eth0 port 12345 >> ${VIDEONAME}.dvs"
echo
echo "press Enter when done"
echo

read

mp4trace -f -s 192.168.0.2 12345 "${VIDEONAME}.mp4" > "${VIDEONAME}.tv" 

echo "Now terminate tcpdump by pressing CTRL-C in its window. "
echo "press Enter when done"

read





