#
# Copyright (c) 2007 Regents of the SIGNET lab, University of Padova.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University of Padova (SIGNET lab) nor the 
#    names of its contributors may be used to endorse or promote products 
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


if {$argc < 3 } {
    puts "usage: $argv0 inputFile outputFile PacketErrorRate \[replicationNumber\]"
    exit 1
}


set opt(infile)  [lindex $argv 0]
set opt(outfile) [lindex $argv 1]
set opt(per)     [lindex $argv 2]

puts "input file         :  $opt(infile)"
puts "output file        :  $opt(outfile)"
puts "packet error rate  :  $opt(per)"

if {$argc >= 4 } {
    set  opt(run)  [lindex $argv 3]
    puts "replication number :  $opt(run)" 
} else {
    set opt(run) 0
}


# Two nodes are created, with only one CBR module 
# per layer. A unidirectional Module/Link connects the two nodes.
# A single CBR flow is started from one node to the other.

load libMiracle.so
load libmiraclelink.so
load libmiraclecbr.so
load libTrace.so
load ../src/.libs/libvid.so

load libTrace.so

proc finish {} {
    global ns tf rm opt
    puts "done!"
    $ns flush-trace
    close $tf
    set per [$rm getper]
    set thr [$rm getthr]
    set rtt [$rm getrtt]
    puts "Packet Error Ratio  :   $per"   
    puts "Throughput          :   $thr"   
    #puts "Round Trip Time     :   $rtt"

    #puts "Tracefile     : $opt(tracefile)"
}



set ns [new Simulator]
$ns use-Miracle


set errorRNG [new RNG]
# seed random number generator according to replication number

for {set j 1} {$j <= $opt(run)} {incr j} {
    $errorRNG next-substream
}



set devnull [open  "/dev/null"  w]

set opt(tracefile)   "out.tr"
set tf [open $opt(tracefile) w]
$ns trace-all $tf


set node1 [$ns create-M_Node] 
set sm [new Module/VID] 
$node1 addModule 2 $sm 0 
$sm setInputFile $opt(infile)



set node2 [$ns create-M_Node]
set rm [new Module/VID]
$node2 addModule 2 $rm 0 
$rm setOutputFile $opt(outfile)


set link [new Module/Link]
$link delay      0.01
$link bandwidth  1000000
$link qsize      1000
$link connect $node1 $sm 1 $node2 $rm 1 


set rv [new RandomVariable/Uniform]
$rv set min_ 0
$rv set max_ 1
$rv use-rng $errorRNG
set em [new ErrorModel]
$em unit pkt
$em set rate_ $opt(per)
$em ranvar $rv

$link addErrorModel $em


set stop 20

$ns at 0 "$sm start"
$ns at 0 "$rm start"
$ns at 20 "$sm stop"
$ns at $stop "$rm stop"
$ns at [expr $stop + 1] "finish; $ns halt"


for {set t 0} {$t < $stop} {set t [expr $t + ($stop / 40.0)  ]} {
    $ns at $t "puts -nonewline . ; flush stdout"
}

puts -nonewline "Simulating"
$ns run
