#
# Copyright (c) 2007 Regents of the SIGNET lab, University of Padova.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University of Padova (SIGNET lab) nor the 
#    names of its contributors may be used to endorse or promote products 
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#



load /home/davide/ns/lib/libMiracle.so
load /home/davide/ns/lib/libmiraclelink.so
load /home/davide/ns/lib/libmiraclecbr.so
load /home/davide/ns/lib/libTrace.so
load /home/davide/ns/modulovideo/lib/libvid.so

load /home/davide/ns/lib/libTrace.so

proc finish {} {
    global ns tf rm opt
    puts "done!"
    $ns flush-trace
    close $tf
    set per [$rm getper]
    set thr [$rm getthr]
    set rtt [$rm getrtt]
    puts "Packet Error Ratio  :   $per"   
    puts "Throughput          :   $thr"   
    #puts "Round Trip Time     :   $rtt"

    puts "Tracefile     : $opt(tracefile)"
}



set ns [new Simulator]
$ns use-Miracle


set errorRNG [new RNG]
# seed random number generator according to replication number
if {$argc >= 1 } {
    set run [lindex $argv 0]
    puts "replication number: $run"
    for {set j 1} {$j < $run} {incr j} {
	$errorRNG next-substream
    }
}




set devnull [open  "/dev/null"  w]

set opt(tracefile)   "out.tr"
set tf [open $opt(tracefile) w]
$ns trace-all $tf


set node1 [$ns create-M_Node] 
set sm [new Module/VID] 
$node1 addModule 2 $sm 0 
$sm setInputFile [lindex $argv 0]



set node2 [$ns create-M_Node]
set rm [new Module/VID]
$node2 addModule 2 $rm 0 
$rm setOutputFile [lindex $argv 1]


set link [new Module/Link]
$link delay      0.01
$link bandwidth  1000000
$link qsize      2500
$link connect $node1 $sm 1 $node2 $rm 1 

set tmp [new ErrorModel/Uniform 0 pkt]
set tmp1 [new ErrorModel/Uniform 1 pkt]

# vettore degli stati (error models)
set m_states [list $tmp $tmp1]
# Durata di ogni stato
set m_periods [list 0.02 0.02]
# Matrice di transizione degli stati

set m_transmx {{0.94737	0.05263}      #packet error rate = 0.05
               {1       0}}           #lunghezza media burst  B = 1 transizione

# m_transmx {{ (1-p)   p   }
#            {   q   (1-q) }}
#
#  B = burstiness
#
#  q = 1/B
#
#  p = (PER*q)/(1-PER)
#

set m_trunit pkt
# usa transizioni a tempo
set m_sttype time
set m_nstates 2
set m_nstart [lindex $m_states 0]
set em [new ErrorModel/MultiState $m_states $m_periods $m_transmx $m_trunit $m_sttype $m_nstates $m_nstart]

$link addErrorModel $em


set stop 20

$ns at 0 "$sm start"
$ns at 0 "$rm start"
$ns at 20 "$sm stop"
$ns at $stop "$rm stop"
$ns at [expr $stop + 1] "finish; $ns halt"


for {set t 0} {$t < $stop} {set t [expr $t + ($stop / 40.0)  ]} {
    $ns at $t "puts -nonewline . ; flush stdout"
}

puts -nonewline "Simulating"
$ns run
