#!/bin/bash
#
# Copyright (c) 2007 Regents of the SIGNET lab, University of Padova.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University of Padova (SIGNET lab) nor the 
#    names of its contributors may be used to endorse or promote products 
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


VIDEONAME=news
FRAME_RATE=25



for tool in MP4Box ffmpeg ns etmp4 psnr
do
    if ! which $tool  &> /dev/null ; then
	echo "Could not find \"$tool\" in current PATH, exiting"
	exit 1
    fi
done



MP4Box -hint -mtu 1024 -fps ${FRAME_RATE} -add ${VIDEONAME}.m4v  ${VIDEONAME}.mp4

ffmpeg -i ${VIDEONAME}.mp4 -y -r ${FRAME_RATE}  ${VIDEONAME}_ref.yuv

ns link_with_errors_vid.tcl ${VIDEONAME}.dvs  ${VIDEONAME}.dvr 0.05 2

etmp4 -p -0 ${VIDEONAME}.dvs ${VIDEONAME}.dvr ${VIDEONAME}.tv ${VIDEONAME}.mp4 ${VIDEONAME}_err

echo
echo "Video with errors: ${VIDEONAME}_err.mp4"
echo

sleep 1

ffmpeg -i ${VIDEONAME}_err.mp4 -y -r ${FRAME_RATE}  ${VIDEONAME}_err.yuv 2>&1 | grep -v 'marker does not match' 


psnr 176 144 420 ${VIDEONAME}_ref.yuv ${VIDEONAME}_err.yuv > ${VIDEONAME}_err_psnr.txt 

echo
echo "PSNR file: ${VIDEONAME}_err_psnr.txt "
echo


if which gnuplot ; then 
    echo " set ylabel \"PSNR\" ; \
       set xlabel \"time (s)\" ; \
        plot \"${VIDEONAME}_err_psnr.txt\" using (\$0/0.04):(\$1) with points pt 3 ps 2" \
	| gnuplot -persist
fi

