/*
 * Copyright (c) 2007 Regents of the SIGNET lab, University of Padova.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Padova (SIGNET lab) nor the 
 *    names of its contributors may be used to endorse or promote products 
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <iostream>
#include <node-core.h>
#include <ip.h>

#include "vid-module.h"

extern packet_t PT_MVID;

int hdr_vid::offset_;

static class VidPktClass : public PacketHeaderClass {
public:
  VidPktClass() : PacketHeaderClass("PacketHeader/VID", sizeof(hdr_vid)) {
    this->bind();
    bind_offset(&hdr_vid::offset_);
  }
} class_vid_pkt; 


static class VidModuleClass : public TclClass {
public:
  VidModuleClass() : TclClass("Module/VID") {}
  TclObject* create(int, const char*const*) {
    return (new VidModule());
  }
} class_module_vid;

void VideoTxHandler::handle(Event* e)
{
  double delay = 0;
  Packet* p = Packet::alloc();
  module->initPkt(p,e);				
  hdr_cmn* ch = hdr_cmn::access(p);
  if (0)                                            // per il debug
    printf("VidModule()::sendPkt, send a pkt (%d)\n", ch->uid());
  module->sendDown(p,delay);
}


int VidModule::uidcnt_;		    // unique id of the packet generated

VidModule::VidModule() 
  : txhandler(this),
    txsn(1),
    hrsn(0),
    pkts_recv(0),
    pkts_ooseq(0),
    pkts_lost(0),
    pkts_last_reset(0),
    rftt(-1),
    lrtime(0),
    sthr(0),
    sumrtt(0),
    rttsamples(0),
    sumftt(0),
    fttsamples(0),
    sumbytes(0),
    sumdt(0)
{
  // binding to TCL variables
  
  bind("destPort_", &dstPort_);
  bind("destAddr_", &dstAddr_);
  bind("alpha_", &alpha_);
  uidcnt_ = 0;
}

VidModule::~VidModule()
{
}

// TCL command interpreter
int VidModule::command(int argc, const char*const* argv)
{
  Tcl& tcl = Tcl::instance();
  if(argc==2)
    {
      if(strcasecmp(argv[1], "start") == 0)	// TCL command to start the transmission
	{
	  start();		
	  return TCL_OK;
	}
      else if(strcasecmp(argv[1], "stop") == 0)	// TCL command to stop the transmission
	{
	  stop();
	  return TCL_OK;
	}
      else if(strcasecmp(argv[1], "getrtt") == 0)	
	{
	  tcl.resultf("%f",GetRTT());
	  return TCL_OK;
	}
      else if(strcasecmp(argv[1], "getftt") == 0)	
	{
	  tcl.resultf("%f",GetFTT());
	  return TCL_OK;
	}
      else if(strcasecmp(argv[1], "getper") == 0)	
	{
	  tcl.resultf("%f",GetPER());
	  return TCL_OK;
	}	
      else if(strcasecmp(argv[1], "getthr") == 0)	
	{
	  tcl.resultf("%f",GetTHR());
	  return TCL_OK;
	}
      else if(strcasecmp(argv[1], "resetStats") == 0)	
	{
	  resetStats();
	  fprintf(stderr,"VidModule::command() resetStats %s, pkts_last_reset=%d, hrsn=%d, txsn=%d\n", tag_ , pkts_last_reset, hrsn, txsn);
	  return TCL_OK;
	}
    }
  if(argc==3)
    {
      if(strcasecmp(argv[1], "setOutputFile") == 0)	
	{
	  if((OutputFile = fopen(argv[2],"w")) != NULL)
	    return TCL_OK;
	  else
	    {
	      cerr  << "VidModule::command(" << argv[1] << ") "
		    << " ERROR: cannot open file " << argv[2]
		    << endl;
	      return TCL_OK;
	    }
	}
      else if(strcasecmp(argv[1], "setInputFile") == 0)	
	{
	  if((InputFile = fopen(argv[2],"r")) != NULL)
	    {		      
		return TCL_OK;
	    }
	}
    }

  return Module::command(argc, argv);
}


int VidModule::crLayCommand(ClMessage* m)
{
  switch (m->type()) 
    {
    default:
      return Module::crLayCommand(m);    
    }  
}

void VidModule::initPkt(Packet* p,Event* e)
{
  hdr_cmn* ch = hdr_cmn::access(p);
  ch->uid() = uidcnt_++;
  ch->ptype() = PT_MVID;
  
  VideoPacket* vp = (VideoPacket*)e;        // "e" in realtà contiene un VideoPacket

  ch->size() = vp->Size;		            // recuperare size da "vp"
				
  hdr_ip* iph = hdr_ip::access(p);
  iph->daddr() = dstAddr_;
  iph->dport() = dstPort_;

  hdr_vid* vidh = HDR_VID(p);
  vidh->sn = txsn;		
  txsn++;
  vidh->video_id = vp->VideoId;		        // recuperare da "vp" il video_id number
  vidh->video_ts = vp->TimeStamp;	        //		              il video_ts
  vidh->ts = Scheduler::instance().clock();	//ts = tempo attuale (non uguale a quello del sender dump)
  ch->timestamp() = vidh->ts;

  if (rftt >= 0)
    {
      vidh->rftt = rftt;
      vidh->rftt_valid = true;
    }
  else
    {
      vidh->rftt_valid = false;
    }

}

void VidModule::start()
{
  if (InputFile){
    long double ts;	    //timestamp del pacchetto
    int id;		        //video_id
    int size;		    //size del payload
    long double to;     //time offset

    fscanf(InputFile,"%Lf",&to);
    rewind(InputFile);
    
    while (fscanf(InputFile, "%Lf IP (tos 0x0, ttl  64, id %d, offset 0, flags [DF], proto: UDP (17), length: %*d) %*d.%*d.%*d.%*d.%*d > %*d.%*d.%*d.%*d.%*d: UDP, length %d",&ts,&id,&size) != EOF){
      VideoPacket* p = new VideoPacket(ts,id,size);
      Scheduler::instance().schedule(&txhandler,p,(ts-to));
    }
  fclose(InputFile);
  }
}



void VidModule::stop()					
{
  return;
}


void VidModule::recv(Packet* p, Handler* h)
{
	hdr_cmn* ch = hdr_cmn::access(p);
  
  recv(p);

}


void VidModule::recv(Packet* p)
{
  hdr_cmn* ch = hdr_cmn::access(p);

  if (debug_ > 1)     
    printf("VidModule(%d)::recv(Packet*p,Handler*) pktId %d\n",getId(), ch->uid());
    
  if (ch->ptype() != PT_MVID)
    {
      drop(p,1,VID_DROP_REASON_UNKNOWN_TYPE);
      incrPktInvalid();
      return;
    }

  hdr_ip* iph = hdr_ip::access(p);
  if (iph->saddr() != dstAddr_)
    {
      drop(p,1,VID_DROP_REASON_WRONG_SADDR);
      incrPktInvalid();
      return;
    }
  if (iph->sport() != dstPort_)
    {
      drop(p,1,VID_DROP_REASON_WRONG_SPORT);
      incrPktInvalid();
      return;
    }

  hdr_vid* vidh = HDR_VID(p);

  int esn = hrsn + 1;           // expected sn
    
  if (vidh->sn < esn)
    { // packet is out of sequence and is to be discarded
      incrPktOoseq();
      if (debug_)
	printf("VidModule::recv() Pkt out of sequence! vidh->sn=%d\thrsn=%d\tesn=%d\n",vidh->sn,hrsn,esn);
      drop(p,1,VID_DROP_REASON_OUT_OF_SEQUENCE);
      return;
    }
  

  /* a new packet has been received */
  incrPktRecv();
  hrsn = vidh->sn;  

  if(OutputFile != NULL){
    if (vidh->sn > esn)
      { // packet losses are observed     
        int lost = (vidh->sn - (esn));      // "lost" = numero di pacchetti persi
        incrPktLost(lost);
	    int i;
        for (i=0 ; i<lost ; i++){
	        fprintf (OutputFile,"\n");      //salta "lost" righe
        }    
      }

    
    long double ots;    // calcolo del timestamp da scrivere sul file di output
    ots = ((Scheduler::instance().clock() - vidh->ts) + vidh->video_ts);
                        //  scrive la riga per il pacchetto ricevuto
    fprintf(OutputFile,"%Lf IP (tos 0x0, ttl  64, id %d, offset 0, flags [DF], proto: UDP (17), length: xxx) xx.xx.xx.xx.xxxx > xx.xx.xx.xx.xxxx: UDP, length %d\n",ots,vidh->video_id,ch->size());
  }

  /* throughput calculation */
  double dt = Scheduler::instance().clock() - lrtime;
  updateThroughput(ch->size(), dt);
  lrtime = Scheduler::instance().clock(); 
  

  /* Forward and Round Trip Time calculation */
  rftt = Scheduler::instance().clock() - vidh->ts;
  if(vidh->rftt_valid) 
    {
      updateFTT(vidh->rftt);
      double rtt = rftt + vidh->rftt;
      updateRTT(rtt);
    }

  Packet::free(p);

  if (pkts_lost + pkts_recv + pkts_last_reset != hrsn)     
    fprintf(stderr,"ERROR VidModule::recv() pkts_lost=%d  pkts_recv=%d  hrsn=%d\n", pkts_lost, pkts_recv, hrsn);
  
}

double VidModule::GetRTT() 
{
  //return srtt;
  return (rttsamples>0) ? sumrtt/rttsamples : 0;
}

double VidModule::GetFTT() 
{
  //  return sftt;
  return (fttsamples>0) ? sumftt/fttsamples : 0;
}

double VidModule::GetPER() 
{
  if ((pkts_recv + pkts_lost)> 0)
    return ((double) pkts_lost / (double)(pkts_recv + pkts_lost));
  else 
    return 0;
}

double VidModule::GetTHR() 
{
  //  return sthr;
  return ((sumdt != 0) ? sumbytes*8/sumdt : 0);
}

void VidModule::updateRTT(double rtt)
{
  srtt = alpha_ * srtt + (1-alpha_) * rtt;
  sumrtt += rtt;
  rttsamples++;
}

void VidModule::updateFTT(double ftt)
{
  sftt = alpha_ * sftt + (1-alpha_) * ftt;
  sumftt += ftt;
  fttsamples++;
}

void VidModule::updateThroughput(int bytes, double dt)
{
  //sthr = alpha_ * sthr + (1-alpha_) * thr;

  sumbytes += bytes;
  sumdt += dt;

  if (debug_) 
    cerr << "bytes=" << bytes << "  dt=" << dt << endl;
}

void VidModule::incrPktLost(int npkts)
{
  pkts_lost += npkts;
}

void VidModule::incrPktRecv()
{
  pkts_recv++;
}

void VidModule::incrPktOoseq()
{
  pkts_ooseq++;
}

void VidModule::incrPktInvalid()
{
  pkts_invalid++;
}


void VidModule::resetStats()
{
  pkts_last_reset += pkts_lost + pkts_recv;
  pkts_recv = 0;
  pkts_ooseq = 0;
  pkts_lost = 0;
  srtt = 0;
  sftt = 0;
  sthr = 0;
  //txsn = 1;
  //hrsn=0;
  rftt = -1;
  sumrtt = 0;
  rttsamples = 0;
  sumftt = 0;
  fttsamples = 0;
  sumbytes = 0;
  sumdt = 0;		
		
}
